@@ -1,7 +1,7 @@
 """
 MIT License
 
-Copyright (c) 2020 Hyeonki Hong <hhk7734@gmail.com>
+Copyright (c) 2020-2021 Hyeonki Hong <hhk7734@gmail.com>
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -21,7 +21,7 @@ LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 SOFTWARE.
 """
-from ctypes import memmove, pointer, set_errno, sizeof
+from ctypes import memmove, memset, pointer, set_errno, sizeof
 from datetime import datetime, timedelta
 from errno import EBUSY, EINVAL, EIO, ENODEV, ENOENT, ENOTTY, EPERM
 from fcntl import ioctl
@@ -42,10 +42,10 @@ from os.path import basename
 import select
 from select import POLLIN, POLLNVAL, POLLPRI
 from stat import S_ISCHR
-from typing import Iterator, List, Union
+from typing import Iterator, List, Optional, Union
 
-from .gpio_h import *
 from .gpiod_h import *
+from ..kernel import *
 
 # core.c
 
@@ -82,7 +82,7 @@ def _is_gpiochip_cdev(path: str) -> bool:
         return False
 
     # Make sure the major and minor numbers of the character device
-    # correspond with the ones in the dev attribute in sysfs.
+    # correspond to the ones in the dev attribute in sysfs.
     devstr = "{}:{}".format(major(statbuf.st_rdev), minor(statbuf.st_rdev))
 
     try:
@@ -98,7 +98,7 @@ def _is_gpiochip_cdev(path: str) -> bool:
     return True
 
 
-def gpiod_chip_open(path: str) -> gpiod_chip:
+def gpiod_chip_open(path: str) -> Optional[gpiod_chip]:
     """
     @brief Open a gpiochip by path.
 
@@ -151,7 +151,7 @@ def gpiod_chip_close(chip: gpiod_chip):
     del chip
 
 
-def gpiod_chip_get_line(chip: gpiod_chip, offset: int) -> gpiod_line:
+def gpiod_chip_get_line(chip: gpiod_chip, offset: int) -> Optional[gpiod_line]:
     """
     @brief Get the handle to the GPIO line at given offset.
 
@@ -178,10 +178,60 @@ def gpiod_chip_get_line(chip: gpiod_chip, offset: int) -> gpiod_line:
     return chip.lines[offset]
 
 
-def _line_maybe_update(line: gpiod_line):
-    status = gpiod_line_update(line)
-    if status < 0:
-        line.up_to_date = False
+def gpiod_line_bias(line: gpiod_line) -> int:
+    """
+    @brief Read the GPIO line bias setting.
+
+    @param line GPIO line object.
+
+    @return Returns GPIOD_LINE_BIAS_PULL_UP, GPIOD_LINE_BIAS_PULL_DOWN,
+            GPIOD_LINE_BIAS_DISABLE or GPIOD_LINE_BIAS_AS_IS.
+    """
+    if line.info_flags & GPIOLINE_FLAG_BIAS_DISABLE:
+        return GPIOD_LINE_BIAS_DISABLE
+    if line.info_flags & GPIOLINE_FLAG_BIAS_PULL_UP:
+        return GPIOD_LINE_BIAS_PULL_UP
+    if line.info_flags & GPIOLINE_FLAG_BIAS_PULL_DOWN:
+        return GPIOD_LINE_BIAS_PULL_DOWN
+
+    return GPIOD_LINE_BIAS_AS_IS
+
+
+def gpiod_line_is_used(line: gpiod_line) -> bool:
+    """
+    @brief Check if the line is currently in use.
+
+    @param line GPIO line object.
+
+    @return True if the line is in use, false otherwise.
+
+    The user space can't know exactly why a line is busy. It may have been
+    requested by another process or hogged by the kernel. It only matters that
+    the line is used and we can't request it.
+    """
+    return bool(line.info_flags & GPIOLINE_FLAG_KERNEL)
+
+
+def gpiod_line_is_open_drain(line: gpiod_line) -> bool:
+    """
+    @brief Check if the line is an open-drain GPIO.
+
+    @param line GPIO line object.
+
+    @return True if the line is an open-drain GPIO, false otherwise.
+    """
+    return bool(line.info_flags & GPIOLINE_FLAG_OPEN_DRAIN)
+
+
+def gpiod_line_is_open_source(line: gpiod_line) -> bool:
+    """
+    @brief Check if the line is an open-source GPIO.
+
+    @param line GPIO line object.
+
+    @return True if the line is an open-source GPIO, false otherwise.
+    """
+    return bool(line.info_flags & GPIOLINE_FLAG_OPEN_SOURCE)
 
 
 def gpiod_line_update(line: gpiod_line) -> int:
@@ -190,12 +240,22 @@ def gpiod_line_update(line: gpiod_line) -> int:
 
     @param line: GPIO line object.
 
-    @return 0 is the operation succeeds. In case of an error this routine
+    @return 0 if the operation succeeds. In case of an error this routine
             returns -1 and sets the last error number.
 
     The line info is initially retrieved from the kernel by
-    gpiod_chip_get_line(). Users can use this line to manually re-read the line
-    info.
+    gpiod_chip_get_line() and is later re-read after every successful request.
+    Users can use this function to manually re-read the line info when needed.
+
+    We currently have no mechanism provided by the kernel for keeping the line
+    info synchronized and for the sake of speed and simplicity of this low-level
+    library we don't want to re-read the line info automatically everytime
+    a property is retrieved. Any daemon using this library must track the state
+    of lines on its own and call this routine if needed.
+
+    The state of requested lines is kept synchronized (or rather cannot be
+    changed by external agents while the ownership of the line is taken) so
+    there's no need to call this function in that case.
     """
     info = gpioline_info()
 
@@ -215,15 +275,11 @@ def gpiod_line_update(line: gpiod_line) -> int:
         if info.flags & GPIOLINE_FLAG_ACTIVE_LOW
         else GPIOD_LINE_ACTIVE_STATE_HIGH
     )
-    line.used = bool(info.flags & GPIOLINE_FLAG_KERNEL)
-    line.open_drain = bool(info.flags & GPIOLINE_FLAG_OPEN_DRAIN)
-    line.open_source = bool(info.flags & GPIOLINE_FLAG_OPEN_SOURCE)
+    line.info_flags = info.flags
 
     line.name = info.name.decode()
     line.consumer = info.consumer.decode()
 
-    line.up_to_date = True
-
     return 0
 
 
@@ -259,6 +315,46 @@ def _line_bulk_all_free(bulk: gpiod_line_bulk) -> bool:
     return True
 
 
+def _line_request_direction_is_valid(direction: int) -> bool:
+    if (
+        direction == GPIOD_LINE_REQUEST_DIRECTION_AS_IS
+        or direction == GPIOD_LINE_REQUEST_DIRECTION_INPUT
+        or direction == GPIOD_LINE_REQUEST_DIRECTION_OUTPUT
+    ):
+        return True
+
+    set_errno(EINVAL)
+    return False
+
+
+def _line_request_direction_to_gpio_handleflag(direction: int) -> int:
+    if direction == GPIOD_LINE_REQUEST_DIRECTION_INPUT:
+        return GPIOHANDLE_REQUEST_INPUT
+    if direction == GPIOD_LINE_REQUEST_DIRECTION_OUTPUT:
+        return GPIOHANDLE_REQUEST_OUTPUT
+
+    return 0
+
+
+def _line_request_flag_to_gpio_handleflag(flags: int) -> int:
+    hflags = 0
+
+    if flags & GPIOD_LINE_REQUEST_FLAG_OPEN_DRAIN:
+        hflags |= GPIOHANDLE_REQUEST_OPEN_DRAIN
+    if flags & GPIOD_LINE_REQUEST_FLAG_OPEN_SOURCE:
+        hflags |= GPIOHANDLE_REQUEST_OPEN_SOURCE
+    if flags & GPIOD_LINE_REQUEST_FLAG_ACTIVE_LOW:
+        hflags |= GPIOHANDLE_REQUEST_ACTIVE_LOW
+    if flags & GPIOD_LINE_REQUEST_FLAG_BIAS_DISABLE:
+        hflags |= GPIOHANDLE_REQUEST_BIAS_DISABLE
+    if flags & GPIOD_LINE_REQUEST_FLAG_BIAS_PULL_DOWN:
+        hflags |= GPIOHANDLE_REQUEST_BIAS_PULL_DOWN
+    if flags & GPIOD_LINE_REQUEST_FLAG_BIAS_PULL_UP:
+        hflags |= GPIOHANDLE_REQUEST_BIAS_PULL_UP
+
+    return hflags
+
+
 def _line_request_values(
     bulk: gpiod_line_bulk,
     config: gpiod_line_request_config,
@@ -283,20 +379,14 @@ def _line_request_values(
     # pylint: disable=no-member
     req = gpiohandle_request()
 
-    if config.flags & GPIOD_LINE_REQUEST_FLAG_OPEN_DRAIN:
-        req.flags |= GPIOHANDLE_REQUEST_OPEN_DRAIN
-    if config.flags & GPIOD_LINE_REQUEST_FLAG_OPEN_SOURCE:
-        req.flags |= GPIOHANDLE_REQUEST_OPEN_SOURCE
-    if config.flags & GPIOD_LINE_REQUEST_FLAG_ACTIVE_LOW:
-        req.flags |= GPIOHANDLE_REQUEST_ACTIVE_LOW
+    req.lines = bulk.num_lines
+    req.flags = _line_request_flag_to_gpio_handleflag(config.flags)
 
     if config.request_type == GPIOD_LINE_REQUEST_DIRECTION_INPUT:
         req.flags |= GPIOHANDLE_REQUEST_INPUT
     elif config.request_type == GPIOD_LINE_REQUEST_DIRECTION_OUTPUT:
         req.flags |= GPIOHANDLE_REQUEST_OUTPUT
 
-    req.lines = bulk.num_lines
-
     for i in range(bulk.num_lines):
         req.lineoffsets[i] = bulk[i].offset
         if (
@@ -314,13 +404,21 @@ def _line_request_values(
     if status < 0:
         return -1
 
+    # line_fd = line_make_fd_handle(req.fd)
     line_fd = line_fd_handle(req.fd)
 
-    for it in bulk:
-        it.state = _LINE_REQUESTED_VALUES
-        # line_set_fd(line, line_fd) -> line_fd_incref(line)
-        it.fd_handle = line_fd
-        _line_maybe_update(it)
+    for i, line in enumerate(bulk):
+        line.state = _LINE_REQUESTED_VALUES
+        line.req_flags = config.flags
+        if config.request_type == GPIOD_LINE_REQUEST_DIRECTION_OUTPUT:
+            line.output_value = req.default_values[i]
+        # line_set_fd(line, line_fd)
+        line.fd_handle = line_fd
+
+        rv = gpiod_line_update(line)
+        if rv:
+            gpiod_line_release_bulk(bulk)
+            return rv
 
     return 0
 
@@ -334,15 +432,9 @@ def _line_request_event_single(
         req.consumer_label = config.consumer[:32].encode()
 
     req.lineoffset = line.offset
+    req.handleflags = _line_request_flag_to_gpio_handleflag(config.flags)
     req.handleflags |= GPIOHANDLE_REQUEST_INPUT
 
-    if config.flags & GPIOD_LINE_REQUEST_FLAG_OPEN_DRAIN:
-        req.flags |= GPIOHANDLE_REQUEST_OPEN_DRAIN
-    if config.flags & GPIOD_LINE_REQUEST_FLAG_OPEN_SOURCE:
-        req.flags |= GPIOHANDLE_REQUEST_OPEN_SOURCE
-    if config.flags & GPIOD_LINE_REQUEST_FLAG_ACTIVE_LOW:
-        req.flags |= GPIOHANDLE_REQUEST_ACTIVE_LOW
-
     if config.request_type == GPIOD_LINE_REQUEST_EVENT_RISING_EDGE:
         req.eventflags |= GPIOEVENT_REQUEST_RISING_EDGE
     elif config.request_type == GPIOD_LINE_REQUEST_EVENT_FALLING_EDGE:
@@ -357,8 +449,14 @@ def _line_request_event_single(
     line_fd = line_fd_handle(req.fd)
 
     line.state = _LINE_REQUESTED_EVENTS
+    line.req_flags = config.flags
+    # line_set_fd(line, line_fd)
     line.fd_handle = line_fd
-    _line_maybe_update(line)
+
+    rv = gpiod_line_update(line)
+    if rv:
+        gpiod_line_release(line)
+        return rv
 
     return 0
 
@@ -572,7 +670,9 @@ def gpiod_line_set_value(line: gpiod_line, value: int) -> int:
     return gpiod_line_set_value_bulk(bulk, [value])
 
 
-def gpiod_line_set_value_bulk(bulk: gpiod_line_bulk, values: List[int]) -> int:
+def gpiod_line_set_value_bulk(
+    bulk: gpiod_line_bulk, values: Optional[List[int]] = None
+) -> int:
     """
     @brief Set the values of a set of GPIO lines.
 
@@ -590,8 +690,11 @@ def gpiod_line_set_value_bulk(bulk: gpiod_line_bulk, values: List[int]) -> int:
     if not _line_bulk_same_chip(bulk) or not _line_bulk_all_requested(bulk):
         return -1
 
-    for i in range(bulk.num_lines):
-        data.values[i] = 1 if values[i] else 0
+    memset(pointer(data), 0, sizeof(data))
+
+    if values is not None:
+        for i in range(bulk.num_lines):
+            data.values[i] = 1 if values[i] else 0
 
     fd = bulk[0].fd_handle.fd
 
@@ -599,9 +702,214 @@ def gpiod_line_set_value_bulk(bulk: gpiod_line_bulk, values: List[int]) -> int:
     if status < 0:
         return -1
 
+    for i, line in enumerate(bulk):
+        line.output_value = data.values[i]
+
     return 0
 
 
+def gpiod_line_set_config(
+    line: gpiod_line, direction: int, flags: int, value: int
+) -> int:
+    """
+    @brief Update the configuration of a single GPIO line.
+
+    @param line:      GPIO line object.
+    @param direction: Updated direction which may be one of
+                      GPIOD_LINE_REQUEST_DIRECTION_AS_IS,
+                      GPIOD_LINE_REQUEST_DIRECTION_INPUT, or
+                      GPIOD_LINE_REQUEST_DIRECTION_OUTPUT.
+    @param flags:     Replacement flags.
+    @param value:     The new output value for the line when direction is
+                      GPIOD_LINE_REQUEST_DIRECTION_OUTPUT.
+
+    @return 0 is the operation succeeds. In case of an error this routine
+            returns -1 and sets the last error number.
+    """
+    bulk = gpiod_line_bulk()
+
+    bulk.add(line)
+
+    return gpiod_line_set_config_bulk(bulk, direction, flags, [value])
+
+
+def gpiod_line_set_config_bulk(
+    bulk: gpiod_line_bulk,
+    direction: int,
+    flags: int,
+    values: Optional[List[int]],
+) -> int:
+    """
+    @brief Update the configuration of a set of GPIO lines.
+
+    @param bulk:      Set of GPIO lines.
+    @param direction: Updated direction which may be one of
+                      GPIOD_LINE_REQUEST_DIRECTION_AS_IS,
+                      GPIOD_LINE_REQUEST_DIRECTION_INPUT, or
+                      GPIOD_LINE_REQUEST_DIRECTION_OUTPUT.
+    @param flags:     Replacement flags.
+    @param values:    An array holding line_bulk->num_lines new logical values
+                      for lines when direction is
+                      GPIOD_LINE_REQUEST_DIRECTION_OUTPUT.
+                      A NULL pointer is interpreted as a logical low for all
+                      lines.
+
+    @return 0 is the operation succeeds. In case of an error this routine
+            returns -1 and sets the last error number.
+
+    If the lines were not previously requested together, the behavior is
+    undefined.
+    """
+    hcfg = gpiohandle_config()
+
+    if not _line_bulk_same_chip(bulk) or not _line_bulk_all_requested(bulk):
+        return -1
+
+    if not _line_request_direction_is_valid(direction):
+        return -1
+
+    memset(pointer(hcfg), 0, sizeof(hcfg))
+
+    hcfg.flags = _line_request_flag_to_gpio_handleflag(flags)
+    hcfg.flags |= _line_request_direction_to_gpio_handleflag(direction)
+    if direction == GPIOD_LINE_REQUEST_DIRECTION_OUTPUT and values is not None:
+        for i in range(bulk.num_lines):
+            hcfg.default_values[i] = 1 if values[i] else 0
+
+    fd = bulk[0].fd_handle.fd
+
+    status = ioctl(fd, GPIOHANDLE_SET_CONFIG_IOCTL, hcfg)
+    if status < 0:
+        return -1
+
+    for i, line in enumerate(bulk):
+        line.req_flags = flags
+        if direction == GPIOD_LINE_REQUEST_DIRECTION_OUTPUT:
+            line.output_value = hcfg.default_values[i]
+
+        rv = gpiod_line_update(line)
+        if rv < 0:
+            return rv
+
+    return 0
+
+
+def gpiod_line_set_flags(line: gpiod_line, flags: int) -> int:
+    """
+    @brief Update the configuration flags of a single GPIO line.
+
+    @param line:  GPIO line object.
+    @param flags: Replacement flags.
+
+    @return 0 is the operation succeeds. In case of an error this routine
+            returns -1 and sets the last error number.
+    """
+    bulk = gpiod_line_bulk()
+
+    bulk.add(line)
+
+    return gpiod_line_set_flags_bulk(bulk, flags)
+
+
+def gpiod_line_set_flags_bulk(bulk: gpiod_line_bulk, flags: int) -> int:
+    """
+    @brief Update the configuration flags of a set of GPIO lines.
+
+    @param bulk:  Set of GPIO lines.
+    @param flags: Replacement flags.
+
+    @return 0 is the operation succeeds. In case of an error this routine
+            returns -1 and sets the last error number.
+
+    If the lines were not previously requested together, the behavior is
+    undefined.
+    """
+    line = bulk[0]
+    values = []
+    direction: int
+
+    if line.direction == GPIOD_LINE_DIRECTION_OUTPUT:
+        for line in bulk:
+            values.append(line.output_value)
+
+        direction = GPIOD_LINE_REQUEST_DIRECTION_OUTPUT
+    else:
+        direction = GPIOD_LINE_REQUEST_DIRECTION_INPUT
+
+    return gpiod_line_set_config_bulk(bulk, direction, flags, values)
+
+
+def gpiod_line_set_direction_input(line: gpiod_line) -> int:
+    """
+    @brief Set the direction of a single GPIO line to input.
+
+    @param line: GPIO line object.
+
+    @return 0 is the operation succeeds. In case of an error this routine
+            returns -1 and sets the last error number.
+    """
+    return gpiod_line_set_config(
+        line, GPIOD_LINE_REQUEST_DIRECTION_INPUT, line.req_flags, 0
+    )
+
+
+def gpiod_line_set_direction_input_bulk(bulk: gpiod_line_bulk) -> int:
+    """
+    @brief Set the direction of a set of GPIO lines to input.
+
+    @param bulk: Set of GPIO lines.
+
+    @return 0 is the operation succeeds. In case of an error this routine
+            returns -1 and sets the last error number.
+
+    If the lines were not previously requested together, the behavior is
+    undefined.
+    """
+    line = bulk[0]
+
+    return gpiod_line_set_config_bulk(
+        bulk, GPIOD_LINE_REQUEST_DIRECTION_INPUT, line.req_flags, None
+    )
+
+
+def gpiod_line_set_direction_output(line: gpiod_line, value: int) -> int:
+    """
+    @brief Set the direction of a single GPIO line to output.
+
+    @param line:  GPIO line object.
+    @param value: The logical value output on the line.
+
+    @return 0 is the operation succeeds. In case of an error this routine
+            returns -1 and sets the last error number.
+    """
+    return gpiod_line_set_config(
+        line, GPIOD_LINE_REQUEST_DIRECTION_OUTPUT, line.req_flags, value
+    )
+
+
+def gpiod_line_set_direction_output_bulk(
+    bulk: gpiod_line_bulk, values: Optional[List[int]]
+) -> int:
+    """
+    @brief Set the direction of a set of GPIO lines to output.
+
+    @param bulk:   Set of GPIO lines.
+    @param values: An array holding line_bulk->num_lines new logical values
+                for lines.  A NULL pointer is interpreted as a logical low
+                for all lines.
+
+    @return 0 is the operation succeeds. In case of an error this routine
+            returns -1 and sets the last error number.
+
+    If the lines were not previously requested together, the behavior is
+    undefined.
+    """
+    line = bulk[0]
+    return gpiod_line_set_config_bulk(
+        bulk, GPIOD_LINE_REQUEST_DIRECTION_OUTPUT, line.req_flags, values
+    )
+
+
 def gpiod_line_event_wait(line: gpiod_line, timeout: timedelta) -> int:
     """
     @brief Wait for an event on a single line.
@@ -620,7 +928,9 @@ def gpiod_line_event_wait(line: gpiod_line, timeout: timedelta) -> int:
 
 
 def gpiod_line_event_wait_bulk(
-    bulk: gpiod_line_bulk, timeout: timedelta, event_bulk: gpiod_line_bulk
+    bulk: gpiod_line_bulk,
+    timeout: timedelta,
+    event_bulk: Optional[gpiod_line_bulk],
 ) -> int:
     """
     @brief Wait for events on a set of lines.
@@ -681,11 +991,11 @@ def gpiod_line_event_read(line: gpiod_line, event: gpiod_line_event) -> int:
 
     @note This function will block if no event was queued for this line.
     """
-    if line.state != _LINE_REQUESTED_EVENTS:
-        set_errno(EPERM)
+    fd = gpiod_line_event_get_fd(line)
+    if fd < 0:
         return -1
 
-    return gpiod_line_event_read_fd(line.fd_handle.fd, event)
+    return gpiod_line_event_read_fd(fd, event)
 
 
 def gpiod_line_event_get_fd(line: gpiod_line) -> int:
@@ -740,7 +1050,7 @@ def gpiod_line_event_read_fd(fd: int, event: gpiod_line_event) -> int:
         else GPIOD_LINE_EVENT_FALLING_EDGE
     )
 
-    sec = evdata.timestamp / 1_000_000_000
+    sec = evdata.timestamp // 1_000_000_000
     event.ts = datetime(year=1970, month=1, day=1) + timedelta(
         days=sec // 86400,
         seconds=sec % 86400,
@@ -753,7 +1063,7 @@ def gpiod_line_event_read_fd(fd: int, event: gpiod_line_event) -> int:
 # helpers.c
 
 
-def gpiod_chip_open_by_name(name: str) -> gpiod_chip:
+def gpiod_chip_open_by_name(name: str) -> Optional[gpiod_chip]:
     """
     @brief Open a gpiochip by name.
 
@@ -764,7 +1074,7 @@ def gpiod_chip_open_by_name(name: str) -> gpiod_chip:
     return gpiod_chip_open("/dev/" + str(name))
 
 
-def gpiod_chip_open_by_number(num: Union[int, str]) -> gpiod_chip:
+def gpiod_chip_open_by_number(num: Union[int, str]) -> Optional[gpiod_chip]:
     """
     @brief Open a gpiochip by number.
 
@@ -775,7 +1085,7 @@ def gpiod_chip_open_by_number(num: Union[int, str]) -> gpiod_chip:
     return gpiod_chip_open("/dev/gpiochip" + str(num))
 
 
-def gpiod_chip_open_by_label(label: str) -> gpiod_chip:
+def gpiod_chip_open_by_label(label: str) -> Optional[gpiod_chip]:
     """
     @brief Open a gpiochip by label.
 
@@ -802,7 +1112,7 @@ def gpiod_chip_open_by_label(label: str) -> gpiod_chip:
     return None
 
 
-def gpiod_chip_open_lookup(descr: Union[int, str]) -> gpiod_chip:
+def gpiod_chip_open_lookup(descr: Union[int, str]) -> Optional[gpiod_chip]:
     """
     @brief Open a gpiochip based on the best guess what the path is.
 
@@ -831,7 +1141,7 @@ def gpiod_chip_open_lookup(descr: Union[int, str]) -> gpiod_chip:
     return chip
 
 
-def gpiod_chip_find_line(chip: gpiod_chip, name: str) -> gpiod_line:
+def gpiod_chip_find_line(chip: gpiod_chip, name: str) -> Optional[gpiod_line]:
     """
     @brief Find a GPIO line by name among lines associated with given GPIO chip.
 
@@ -865,7 +1175,7 @@ class gpiod_chip_iter:
         self.chips = []
         self.offset = 0
 
-    def __iter__(self):
+    def __iter__(self) -> Iterator[gpiod_chip]:
         """
         gpiod_chip_iter_new()
 
