@@ -1,7 +1,7 @@
 """
 MIT License
 
-Copyright (c) 2020 Hyeonki Hong <hhk7734@gmail.com>
+Copyright (c) 2020-2021 Hyeonki Hong <hhk7734@gmail.com>
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -21,6 +21,8 @@ LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 SOFTWARE.
 """
+from __future__ import annotations
+
 from copy import copy
 from ctypes import get_errno
 from datetime import timedelta
@@ -32,42 +34,24 @@ from .. import libgpiod
 
 # pylint: disable=too-many-lines
 
+_CHIP_OPEN_LOOKUP = 1
+_CHIP_OPEN_BY_PATH = 2
+_CHIP_OPEN_BY_NAME = 3
+_CHIP_OPEN_BY_LABEL = 4
+_CHIP_OPEN_BY_NUMBER = 5
 
-class chip:
-    # pylint: disable=too-few-public-methods
-    OPEN_LOOKUP = 1
-    OPEN_BY_PATH = 2
-    OPEN_BY_NAME = 3
-    OPEN_BY_LABEL = 4
-    OPEN_BY_NUMBER = 5
-
-
-class line:
-    # pylint: disable=too-few-public-methods
-    pass
-
-
-class line_bulk:
-    # pylint: disable=too-few-public-methods
-    pass
-
-
-class line_event:
-    # pylint: disable=too-few-public-methods
-    pass
-
-
-class line_request:
-    # pylint: disable=too-few-public-methods
-    pass
+_LINE_BIAS_AS_IS = 1
+_LINE_BIAS_DISABLE = 2
+_LINE_BIAS_PULL_UP = 3
+_LINE_BIAS_PULL_DOWN = 4
 
 
 open_funcs = {
-    chip.OPEN_LOOKUP: libgpiod.gpiod_chip_open_lookup,
-    chip.OPEN_BY_PATH: libgpiod.gpiod_chip_open,
-    chip.OPEN_BY_NAME: libgpiod.gpiod_chip_open_by_name,
-    chip.OPEN_BY_LABEL: libgpiod.gpiod_chip_open_by_label,
-    chip.OPEN_BY_NUMBER: libgpiod.gpiod_chip_open_by_number,
+    _CHIP_OPEN_LOOKUP: libgpiod.gpiod_chip_open_lookup,
+    _CHIP_OPEN_BY_PATH: libgpiod.gpiod_chip_open,
+    _CHIP_OPEN_BY_NAME: libgpiod.gpiod_chip_open_by_name,
+    _CHIP_OPEN_BY_LABEL: libgpiod.gpiod_chip_open_by_label,
+    _CHIP_OPEN_BY_NUMBER: libgpiod.gpiod_chip_open_by_number,
 }
 
 
@@ -97,7 +81,7 @@ class chip:
     def __init__(
         self,
         device: Optional[Union[int, str]] = None,
-        how: int = chip.OPEN_LOOKUP,
+        how: int = _CHIP_OPEN_LOOKUP,
         chip_shared: Optional[shared_chip] = None,
     ) -> None:
         """
@@ -129,7 +113,7 @@ class chip:
         """
 
     def open(
-        self, device: Union[int, str], how: int = chip.OPEN_LOOKUP
+        self, device: Union[int, str], how: int = _CHIP_OPEN_LOOKUP
     ) -> None:
         """
         @brief Open a GPIO chip.
@@ -346,11 +330,11 @@ class chip:
         """
         return self._m_chip.get() is not None
 
-    OPEN_LOOKUP = 1
-    OPEN_BY_PATH = 2
-    OPEN_BY_NAME = 3
-    OPEN_BY_LABEL = 4
-    OPEN_BY_NUMBER = 5
+    OPEN_LOOKUP = _CHIP_OPEN_LOOKUP
+    OPEN_BY_PATH = _CHIP_OPEN_BY_PATH
+    OPEN_BY_NAME = _CHIP_OPEN_BY_NAME
+    OPEN_BY_LABEL = _CHIP_OPEN_BY_LABEL
+    OPEN_BY_NUMBER = _CHIP_OPEN_BY_NUMBER
 
     def _throw_if_noref_and_get_m_chip(self) -> libgpiod.gpiod_chip:
         _m_chip_get = self._m_chip.get()
@@ -369,9 +353,12 @@ class line_request:
     EVENT_RISING_EDGE = 5
     EVENT_BOTH_EDGES = 6
 
-    FLAG_ACTIVE_LOW = 0b001
-    FLAG_OPEN_SOURCE = 0b010
-    FLAG_OPEN_DRAIN = 0b100
+    FLAG_ACTIVE_LOW = libgpiod.GPIOD_BIT(0)
+    FLAG_OPEN_SOURCE = libgpiod.GPIOD_BIT(1)
+    FLAG_OPEN_DRAIN = libgpiod.GPIOD_BIT(2)
+    FLAG_BIAS_DISABLE = libgpiod.GPIOD_BIT(3)
+    FLAG_BIAS_PULL_DOWN = libgpiod.GPIOD_BIT(4)
+    FLAG_BIAS_PULL_UP = libgpiod.GPIOD_BIT(5)
 
     def __init__(self):
         self.consumer = ""
@@ -390,9 +377,20 @@ reqtype_mapping = {
 }
 
 reqflag_mapping = {
+    # pylint: disable=line-too-long
     line_request.FLAG_ACTIVE_LOW: libgpiod.GPIOD_LINE_REQUEST_FLAG_ACTIVE_LOW,
     line_request.FLAG_OPEN_DRAIN: libgpiod.GPIOD_LINE_REQUEST_FLAG_OPEN_DRAIN,
     line_request.FLAG_OPEN_SOURCE: libgpiod.GPIOD_LINE_REQUEST_FLAG_OPEN_SOURCE,
+    line_request.FLAG_BIAS_DISABLE: libgpiod.GPIOD_LINE_REQUEST_FLAG_BIAS_DISABLE,
+    line_request.FLAG_BIAS_PULL_DOWN: libgpiod.GPIOD_LINE_REQUEST_FLAG_BIAS_PULL_DOWN,
+    line_request.FLAG_BIAS_PULL_UP: libgpiod.GPIOD_LINE_REQUEST_FLAG_BIAS_PULL_UP,
+}
+
+bias_mapping = {
+    libgpiod.GPIOD_LINE_BIAS_PULL_UP: _LINE_BIAS_PULL_UP,
+    libgpiod.GPIOD_LINE_BIAS_PULL_DOWN: _LINE_BIAS_PULL_DOWN,
+    libgpiod.GPIOD_LINE_BIAS_DISABLE: _LINE_BIAS_DISABLE,
+    libgpiod.GPIOD_LINE_BIAS_AS_IS: _LINE_BIAS_AS_IS,
 }
 
 
@@ -492,6 +490,19 @@ class line:
         )
 
     @property
+    def bias(self) -> int:
+        """
+        @brief Get current bias of this line.
+
+        @return Current bias setting.
+
+        Usage:
+            print(line.bias == line.BIAS_PULL_UP)
+        """
+        return bias_mapping[
+            libgpiod.gpiod_line_bias(self._throw_if_null_and_get_m_line())
+        ]
+
     def is_used(self) -> bool:
         """
         @brief Check if this line is used by the kernel or other user space
@@ -500,11 +511,10 @@ class line:
         @return True if this line is in use, false otherwise.
 
         Usage:
-            print(line.is_used)
+            print(line.is_used())
         """
-        return self._throw_if_null_and_get_m_line().used
+        return libgpiod.gpiod_line_is_used(self._throw_if_null_and_get_m_line())
 
-    @property
     def is_open_drain(self) -> bool:
         """
         @brief Check if this line represents an open-drain GPIO.
@@ -512,11 +522,12 @@ class line:
         @return True if the line is an open-drain GPIO, false otherwise.
 
         Usage:
-            print(line.is_open_drain)
+            print(line.is_open_drain())
         """
-        return self._throw_if_null_and_get_m_line().open_drain
+        return libgpiod.gpiod_line_is_open_drain(
+            self._throw_if_null_and_get_m_line()
+        )
 
-    @property
     def is_open_source(self) -> bool:
         """
         @brief Check if this line represents an open-source GPIO.
@@ -524,9 +535,11 @@ class line:
         @return True if the line is an open-source GPIO, false otherwise.
 
         Usage:
-            print(line.is_open_source)
+            print(line.is_open_source())
         """
-        return self._throw_if_null_and_get_m_line().open_source
+        return libgpiod.gpiod_line_is_open_source(
+            self._throw_if_null_and_get_m_line()
+        )
 
     def request(self, config: line_request, default_val: int = 0):
         """
@@ -550,6 +563,10 @@ class line:
         conf.request_type = reqtype_mapping[config.request_type]
         conf.flags = 0
 
+        for k, v in reqflag_mapping.items():
+            if config.flags & k:
+                conf.flags |= v
+
         rv = libgpiod.gpiod_line_request(_m_line, conf, default_val)
         if rv:
             errno = get_errno()
@@ -564,7 +581,6 @@ class line:
         """
         libgpiod.gpiod_line_release(self._throw_if_null_and_get_m_line())
 
-    @property
     def is_requested(self) -> bool:
         """
         @brief Check if this user has ownership of this line.
@@ -572,7 +588,7 @@ class line:
         @return True if the user has ownership of this line, false otherwise.
 
         Usage:
-            print(line.is_requested)
+            print(line.is_requested())
         """
         return libgpiod.gpiod_line_is_requested(
             self._throw_if_null_and_get_m_line()
@@ -614,6 +630,55 @@ class line:
                 errno, strerror(errno), "error setting GPIO line value"
             )
 
+    def set_config(self, direction: int, flags: int, value: int = 0):
+        """
+        @brief Set configuration of this line.
+
+        @param direction: New direction.
+        @param flags:     Replacement flags.
+        @param value:     New value (0 or 1) - only matters for OUTPUT
+                          direction.
+        """
+        self._throw_if_null()
+
+        bulk = line_bulk([self])
+
+        bulk.set_config(direction, flags, [value])
+
+    def set_flags(self, flags: int):
+        """
+        @brief Set configuration flags of this line.
+
+        @param flags: Replacement flags.
+        """
+        self._throw_if_null()
+
+        bulk = line_bulk([self])
+
+        bulk.set_flags(flags)
+
+    def set_direction_input(self):
+        """
+        @brief Change the direction this line to input.
+        """
+        self._throw_if_null()
+
+        bulk = line_bulk([self])
+
+        bulk.set_direction_input()
+
+    def set_direction_output(self, value: int = 0):
+        """
+        @brief Change the direction this lines to output.
+
+        @param value: New value (0 or 1).
+        """
+        self._throw_if_null()
+
+        bulk = line_bulk([self])
+
+        bulk.set_direction_output([value])
+
     def event_wait(self, timeout: timedelta) -> bool:
         """
         @brief Wait for an event on this line.
@@ -707,6 +772,21 @@ class line:
         """
         return self._m_chip
 
+    def update(self) -> None:
+        """
+        @brief Re-read the line info from the kernel.
+
+        Usage:
+            line.update()
+        """
+        ret = libgpiod.gpiod_line_update(self._throw_if_null_and_get_m_line())
+
+        if ret < 0:
+            errno = get_errno()
+            raise OSError(
+                errno, strerror(errno), "unable to update the line info"
+            )
+
     def reset(self):
         """
         @brief Reset the state of this object.
@@ -765,6 +845,15 @@ class line:
     ACTIVE_LOW = 1
     ACTIVE_HIGH = 2
 
+    BIAS_AS_IS = 1
+    BIAS_DISABLE = 2
+    BIAS_PULL_UP = 3
+    BIAS_PULL_DOWN = 4
+
+    def _throw_if_null(self) -> None:
+        if self._m_line is None:
+            raise RuntimeError("object not holding a GPIO line handle")
+
     def _throw_if_null_and_get_m_line(self) -> libgpiod.gpiod_line:
         if self._m_line is None:
             raise RuntimeError("object not holding a GPIO line handle")
@@ -932,7 +1021,7 @@ class line_bulk:
         if self.size != len(default_vals):
             raise ValueError(
                 "the number of default values must correspond "
-                "with the number of lines"
+                "to the number of lines"
             )
 
         try:
@@ -959,7 +1048,7 @@ class line_bulk:
         @brief Read values from all lines held by this object.
 
         @return List containing line values the order of which corresponds
-                with the order of lines in the internal array.
+                to the order of lines in the internal array.
 
         Usage:
             ret = bulk.get_values()
@@ -986,13 +1075,125 @@ class line_bulk:
 
         if self.size != len(values):
             raise ValueError(
-                "the size of values array must correspond with "
+                "the size of values array must correspond to "
                 "the number of lines"
             )
 
         for i in range(self.size):
             self._m_bulk[i].set_value(values[i])
 
+    def set_config(
+        self, direction: int, flags: int, values: Optional[List[int]] = None
+    ):
+        """
+        @brief Set configuration of all lines held by this object.
+
+        @param direction: New direction.
+        @param flags:     Replacement flags.
+
+        @param List of values to set. Must be the same size as the number of
+               lines held by this line_bulk.
+               Only relevant for output direction requests.
+        """
+        self._throw_if_empty()
+
+        if values is not None and self.size != len(values):
+            raise ValueError(
+                "the size of values array must correspond to "
+                "the number of lines"
+            )
+
+        gflags = 0
+
+        for first, second in reqflag_mapping.items():
+            if first & flags:
+                gflags |= second
+
+        bulk = libgpiod.gpiod_line_bulk()
+
+        self._to_line_bulk(bulk)
+
+        rv = libgpiod.gpiod_line_set_config_bulk(
+            bulk, direction, gflags, values
+        )
+        if rv < 0:
+            errno = get_errno()
+            raise OSError(
+                errno, strerror(errno), "error setting GPIO line config"
+            )
+
+    def set_flags(self, flags: int):
+        """
+        @brief Set configuration flags of all lines held by this object.
+
+        @param flags: Replacement flags.
+        """
+        self._throw_if_empty()
+
+        bulk = libgpiod.gpiod_line_bulk()
+
+        self._to_line_bulk(bulk)
+
+        gflags = 0
+
+        for first, second in reqflag_mapping.items():
+            if first & flags:
+                gflags |= second
+
+        rv = libgpiod.gpiod_line_set_flags_bulk(bulk, gflags)
+        if rv < 0:
+            errno = get_errno()
+            raise OSError(
+                errno, strerror(errno), "error setting GPIO line flags"
+            )
+
+    def set_direction_input(self):
+        """
+        @brief Change the direction all lines held by this object to input.
+        """
+        self._throw_if_empty()
+
+        bulk = libgpiod.gpiod_line_bulk()
+
+        self._to_line_bulk(bulk)
+
+        rv = libgpiod.gpiod_line_set_direction_input_bulk(bulk)
+        if rv < 0:
+            errno = get_errno()
+            raise OSError(
+                errno,
+                strerror(errno),
+                "error setting GPIO line direction to input",
+            )
+
+    def set_direction_output(self, values: Optional[List[int]] = None):
+        """
+        @brief Change the direction this lines to output.
+
+        @param values: Vector of values to set. Must be the same size as the
+                       number of lines held by this line_bulk.
+        """
+        self._throw_if_empty()
+
+        if values is not None and self.size != len(values):
+            raise ValueError(
+                "the size of values array must correspond to "
+                "the number of lines"
+            )
+
+        bulk = libgpiod.gpiod_line_bulk()
+
+        self._to_line_bulk(bulk)
+
+        rv = libgpiod.gpiod_line_set_direction_output_bulk(bulk, values)
+        if rv < 0:
+            errno = get_errno()
+            raise OSError(
+                errno,
+                strerror(errno),
+                "error setting GPIO line direction to output",
+            )
+
     def event_wait(self, timeout: timedelta) -> line_bulk:
         """
         @brief Poll the set of lines for line events.
